summaryrefslogtreecommitdiff
path: root/benchmarks/CUDA/BlackScholes/BlackScholes_kernel.cuh
diff options
context:
space:
mode:
Diffstat (limited to 'benchmarks/CUDA/BlackScholes/BlackScholes_kernel.cuh')
-rw-r--r--benchmarks/CUDA/BlackScholes/BlackScholes_kernel.cuh122
1 files changed, 122 insertions, 0 deletions
diff --git a/benchmarks/CUDA/BlackScholes/BlackScholes_kernel.cuh b/benchmarks/CUDA/BlackScholes/BlackScholes_kernel.cuh
new file mode 100644
index 0000000..d99f69a
--- /dev/null
+++ b/benchmarks/CUDA/BlackScholes/BlackScholes_kernel.cuh
@@ -0,0 +1,122 @@
+/*
+ * Copyright 1993-2007 NVIDIA Corporation. All rights reserved.
+ *
+ * NOTICE TO USER:
+ *
+ * This source code is subject to NVIDIA ownership rights under U.S. and
+ * international Copyright laws. Users and possessors of this source code
+ * are hereby granted a nonexclusive, royalty-free license to use this code
+ * in individual and commercial software.
+ *
+ * NVIDIA MAKES NO REPRESENTATION ABOUT THE SUITABILITY OF THIS SOURCE
+ * CODE FOR ANY PURPOSE. IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR
+ * IMPLIED WARRANTY OF ANY KIND. NVIDIA DISCLAIMS ALL WARRANTIES WITH
+ * REGARD TO THIS SOURCE CODE, INCLUDING ALL IMPLIED WARRANTIES OF
+ * MERCHANTABILITY, NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
+ * IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL,
+ * OR CONSEQUENTIAL DAMAGES, OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS
+ * OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
+ * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE
+ * OR PERFORMANCE OF THIS SOURCE CODE.
+ *
+ * U.S. Government End Users. This source code is a "commercial item" as
+ * that term is defined at 48 C.F.R. 2.101 (OCT 1995), consisting of
+ * "commercial computer software" and "commercial computer software
+ * documentation" as such terms are used in 48 C.F.R. 12.212 (SEPT 1995)
+ * and is provided to the U.S. Government only as a commercial end item.
+ * Consistent with 48 C.F.R.12.212 and 48 C.F.R. 227.7202-1 through
+ * 227.7202-4 (JUNE 1995), all U.S. Government End Users acquire the
+ * source code with only those rights set forth herein.
+ *
+ * Any use of this source code in individual and commercial software must
+ * include, in the user documentation and internal comments to the code,
+ * the above Disclaimer and U.S. Government End Users Notice.
+ */
+
+
+
+///////////////////////////////////////////////////////////////////////////////
+// Polynomial approximation of cumulative normal distribution function
+///////////////////////////////////////////////////////////////////////////////
+__device__ inline float cndGPU(float d){
+ const float A1 = 0.31938153f;
+ const float A2 = -0.356563782f;
+ const float A3 = 1.781477937f;
+ const float A4 = -1.821255978f;
+ const float A5 = 1.330274429f;
+ const float RSQRT2PI = 0.39894228040143267793994605993438f;
+
+ float
+ K = 1.0f / (1.0f + 0.2316419f * fabsf(d));
+
+ float
+ cnd = RSQRT2PI * __expf(- 0.5f * d * d) *
+ (K * (A1 + K * (A2 + K * (A3 + K * (A4 + K * A5)))));
+
+ if(d > 0)
+ cnd = 1.0f - cnd;
+
+ return cnd;
+}
+
+
+///////////////////////////////////////////////////////////////////////////////
+// Black-Scholes formula for both call and put
+///////////////////////////////////////////////////////////////////////////////
+__device__ inline void BlackScholesBodyGPU(
+ float& CallResult,
+ float& PutResult,
+ float S, //Stock price
+ float X, //Option strike
+ float T, //Option years
+ float R, //Riskless rate
+ float V //Volatility rate
+){
+ float sqrtT, expRT;
+ float d1, d2, CNDD1, CNDD2;
+
+ sqrtT = sqrtf(T);
+ d1 = (__logf(S / X) + (R + 0.5f * V * V) * T) / (V * sqrtT);
+ d2 = d1 - V * sqrtT;
+
+ CNDD1 = cndGPU(d1);
+ CNDD2 = cndGPU(d2);
+
+ //Calculate Call and Put simultaneously
+ expRT = __expf(- R * T);
+ CallResult = S * CNDD1 - X * expRT * CNDD2;
+ PutResult = X * expRT * (1.0f - CNDD2) - S * (1.0f - CNDD1);
+}
+
+
+////////////////////////////////////////////////////////////////////////////////
+//Process an array of optN options on GPU
+////////////////////////////////////////////////////////////////////////////////
+__global__ void BlackScholesGPU(
+ float *d_CallResult,
+ float *d_PutResult,
+ float *d_StockPrice,
+ float *d_OptionStrike,
+ float *d_OptionYears,
+ float Riskfree,
+ float Volatility,
+ int optN
+){
+ //Thread index
+ const int tid = blockDim.x * blockIdx.x + threadIdx.x;
+ //Total number of threads in execution grid
+ const int THREAD_N = blockDim.x * gridDim.x;
+
+ //No matter how small is execution grid or how large OptN is,
+ //exactly OptN indices will be processed with perfect memory coalescing
+ for(int opt = tid; opt < optN; opt += THREAD_N)
+ BlackScholesBodyGPU(
+ d_CallResult[opt],
+ d_PutResult[opt],
+ d_StockPrice[opt],
+ d_OptionStrike[opt],
+ d_OptionYears[opt],
+ Riskfree,
+ Volatility
+ );
+}